@@ -20,15 +20,6 @@
 #include "xtensor/xexpression.hpp"
 #include "xtensor/xiterator.hpp"
 
-// DETECT 3.6 <= clang < 3.8 for compiler bug workaround.
-#ifdef __clang__
-    #if __clang_major__ == 3 && __clang_minor__ < 8
-        #define X_OLD_CLANG
-        #include <initializer_list>
-        #include <vector>
-    #endif
-#endif
-
 namespace xt
 {
 
@@ -59,34 +50,41 @@ namespace xt
      * to a specified shape. xbroadcast is not meant to be used directly, but
      * only with the \ref broadcast helper functions.
      *
-     * @tparam E the type of the \ref xexpression to broadcast
-     * @tparam S the type of the specified shape.
+     * @tparam CT the closure type of the \ref xexpression to broadcast
+     * @tparam X the type of the specified shape.
+     *
+     * @sa broadcast
      */
-    template <class E, class X, bool LV>
-    class xbroadcast : public xexpression<xbroadcast<E, X, LV>>
+    template <class CT, class X>
+    class xbroadcast : public xexpression<xbroadcast<CT, X>>
     {
 
     public:
 
-        using self_type = xbroadcast<E, X, LV>;
+        using self_type = xbroadcast<CT, X>;
+        using xexpression_type = std::decay_t<CT>;
 
-        using value_type = typename E::value_type;
-        using reference = typename E::reference;
-        using const_reference = typename E::const_reference;
-        using pointer = typename E::pointer;
-        using const_pointer = typename E::const_pointer;
-        using size_type = typename E::size_type;
-        using difference_type = typename E::difference_type;
+        using value_type = typename xexpression_type::value_type;
+        using reference = typename xexpression_type::reference;
+        using const_reference = typename xexpression_type::const_reference;
+        using pointer = typename xexpression_type::pointer;
+        using const_pointer = typename xexpression_type::const_pointer;
+        using size_type = typename xexpression_type::size_type;
+        using difference_type = typename xexpression_type::difference_type;
         
-        using shape_type = promote_shape_t<typename E::shape_type, X>;
-        using closure_type = const self_type;
+        using shape_type = promote_shape_t<typename xexpression_type::shape_type, X>;
+
+        using const_stepper = typename xexpression_type::const_stepper;
+        using stepper = const_stepper;
 
-        using const_stepper = typename E::const_stepper;
         using const_iterator = xiterator<const_stepper, shape_type>;
+        using iterator = const_iterator;
+
         using const_storage_iterator = const_iterator;
+        using storage_iterator = const_storage_iterator;
 
         template <class S>
-        xbroadcast(const E& e, S s) noexcept;
+        xbroadcast(CT e, S&& s) noexcept;
 
         size_type dimension() const noexcept;
         const shape_type & shape() const noexcept;
@@ -131,7 +129,7 @@ namespace xt
 
     private:
 
-        std::conditional_t<LV, const E&, E> m_e;
+        CT m_e;
         shape_type m_shape;
     };
 
@@ -139,46 +137,6 @@ namespace xt
      * broadcast implementation *
      ****************************/
 
-    namespace detail
-    {
-        template <class R, class A>
-        struct shape_forwarder
-        {
-            static inline R run(const A& r)
-            {
-                return R(std::begin(r), std::end(r));
-            }
-        };
-
-        template <class I, std::size_t L, class A>
-        struct shape_forwarder<std::array<I, L>, A>
-        {
-            using R = std::array<I, L>;
-
-            static inline R run(const A& r)
-            {
-                R ret;
-                std::copy(std::begin(r), std::end(r), std::begin(ret));
-                return ret;
-            }
-        };
-
-        template <class R>
-        struct shape_forwarder<R, R>
-        {
-            static inline const R& run(const R& r)
-            {
-                return r;
-            }
-        };
-
-        template <class R, class A>
-        inline auto forward_shape(const A& s)
-        {
-            return shape_forwarder<R, A>::run(s);
-        }
-    }
-
     /**
      * @brief Returns an \ref xexpression broadcasting the given expression to
      * a specified shape.
@@ -192,27 +150,26 @@ namespace xt
     template <class E, class S>
     inline auto broadcast(E&& e, const S& s) noexcept
     {
-        constexpr bool is_lvalue = std::is_lvalue_reference<decltype(e)>::value;
-        using broadcast_type = xbroadcast<get_xexpression_type<E>, S, is_lvalue>;
+        using broadcast_type = xbroadcast<const_xclosure_t<E>, S>;
         using shape_type = typename broadcast_type::shape_type;
-        return broadcast_type(std::forward<E>(e), detail::forward_shape<shape_type>(s));
+        return broadcast_type(std::forward<E>(e), forward_sequence<shape_type>(s));
     }
 
 #ifdef X_OLD_CLANG
     template <class E, class I>
     inline auto broadcast(E&& e, std::initializer_list<I> s) noexcept
     {
-        using broadcast_type = xbroadcast<get_xexpression_type<E>, std::vector<std::size_t>, false>;
+        using broadcast_type = xbroadcast<const_xclosure_t<E>, std::vector<std::size_t>>;
         using shape_type = typename broadcast_type::shape_type;
-        return broadcast_type(std::forward<E>(e), detail::forward_shape<shape_type>(s));
+        return broadcast_type(std::forward<E>(e), forward_sequence<shape_type>(s));
     }
 #else
     template <class E, class I, std::size_t L>
     inline auto broadcast(E&& e, const I(&s)[L]) noexcept
     {
-        using broadcast_type = xbroadcast<get_xexpression_type<E>, std::array<std::size_t, L>, false>;
+        using broadcast_type = xbroadcast<const_xclosure_t<E>, std::array<std::size_t, L>>;
         using shape_type = typename broadcast_type::shape_type;
-        return broadcast_type(std::forward<E>(e), detail::forward_shape<shape_type>(s));
+        return broadcast_type(std::forward<E>(e), forward_sequence<shape_type>(s));
     }
 #endif
 
@@ -231,10 +188,10 @@ namespace xt
      * @param e the expression to broadcast
      * @param s the shape to apply
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline xbroadcast<E, X, LV>::xbroadcast(const E& e, S s) noexcept
-        : m_e(e), m_shape(std::move(s))
+    inline xbroadcast<CT, X>::xbroadcast(CT e, S&& s) noexcept
+        : m_e(e), m_shape(std::forward<S>(s))
     {
         xt::broadcast_shape(e.shape(), m_shape);
     }
@@ -247,8 +204,8 @@ namespace xt
     /**
      * Returns the number of dimensions of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::dimension() const noexcept -> size_type
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::dimension() const noexcept -> size_type
     {
         return m_shape.size();
     }
@@ -256,8 +213,8 @@ namespace xt
     /**
      * Returns the shape of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::shape() const noexcept -> const shape_type &
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::shape() const noexcept -> const shape_type &
     {
         return m_shape;
     }
@@ -272,9 +229,9 @@ namespace xt
      * must be unsigned integers, the number of indices should be equal or greater than
      * the number of dimensions of the expression.
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class... Args>
-    inline auto xbroadcast<E, X, LV>::operator()(Args... args) const -> const_reference
+    inline auto xbroadcast<CT, X>::operator()(Args... args) const -> const_reference
     {
         return detail::get_element(m_e, args...);
     }
@@ -285,8 +242,8 @@ namespace xt
      * must be unsigned integers, the number of indices in the sequence should be equal or greater
      * than the number of dimensions of the container.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::operator[](const xindex& index) const -> const_reference
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::operator[](const xindex& index) const -> const_reference
     {
         return element(index.cbegin(), index.cend());
     }
@@ -294,13 +251,13 @@ namespace xt
     /**
      * Returns a constant reference to the element at the specified position in the expression.
      * @param first iterator starting the sequence of indices
-     * @param second iterator starting the sequence of indices
-     * The number of indices in the squence should be equal or greater
+     * @param last iterator ending the sequence of indices
+     * The number of indices in the squence should be equal to or greater
      * than the number of dimensions of the function.
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class It>
-    inline auto xbroadcast<E, X, LV>::element(It, It last) const -> const_reference
+    inline auto xbroadcast<CT, X>::element(It, It last) const -> const_reference
     {
         // Workaround MSVC bug. m_e.element(last - dimension(), last) does not build.
         It first = last;
@@ -318,9 +275,9 @@ namespace xt
      * @param shape the result shape
      * @return a boolean indicating whether the broadcasting is trivial
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline bool xbroadcast<E, X, LV>::broadcast_shape(S& shape) const
+    inline bool xbroadcast<CT, X>::broadcast_shape(S& shape) const
     { 
         return xt::broadcast_shape(m_shape, shape);
     }
@@ -330,9 +287,9 @@ namespace xt
      * the broadcasting is trivial.
      * @return a boolean indicating whether the broadcasting is trivial
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline bool xbroadcast<E, X, LV>::is_trivial_broadcast(const S& strides) const noexcept
+    inline bool xbroadcast<CT, X>::is_trivial_broadcast(const S& strides) const noexcept
     {
         return dimension() == m_e.dimension() &&
                std::equal(m_shape.cbegin(), m_shape.cend(), m_e.shape().cbegin()) &&
@@ -347,8 +304,8 @@ namespace xt
     /**
      * Returns a constant iterator to the first element of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::begin() const noexcept -> const_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::begin() const noexcept -> const_iterator
     {
         return cxbegin(shape());
     }
@@ -357,8 +314,8 @@ namespace xt
      * Returns a constant iterator to the element following the last element
      * of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::end() const noexcept -> const_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::end() const noexcept -> const_iterator
     {
         return cxend(shape());
     }
@@ -366,8 +323,8 @@ namespace xt
     /**
      * Returns a constant iterator to the first element of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::cbegin() const noexcept -> const_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::cbegin() const noexcept -> const_iterator
     {
         return cxbegin(shape());
     }
@@ -376,8 +333,8 @@ namespace xt
      * Returns a constant iterator to the element following the last element
      * of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::cend() const noexcept -> const_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::cend() const noexcept -> const_iterator
     {
         return cxend(shape());
     }
@@ -387,9 +344,9 @@ namespace xt
      * iteration is broadcasted to the specified shape.
      * @param shape the shape used for braodcasting
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline auto xbroadcast<E, X, LV>::xbegin(const S& shape) const noexcept -> xiterator<const_stepper, S>
+    inline auto xbroadcast<CT, X>::xbegin(const S& shape) const noexcept -> xiterator<const_stepper, S>
     {
         // Could check if (broadcastable(shape, m_shape)
         return xiterator<const_stepper, S>(stepper_begin(shape), shape);
@@ -400,9 +357,9 @@ namespace xt
      * expression. The iteration is broadcasted to the specified shape.
      * @param shape the shape used for broadcasting
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline auto xbroadcast<E, X, LV>::xend(const S& shape) const noexcept -> xiterator<const_stepper, S>
+    inline auto xbroadcast<CT, X>::xend(const S& shape) const noexcept -> xiterator<const_stepper, S>
     {
         // Could check if (broadcastable(shape, m_shape)
         return xiterator<const_stepper, S>(stepper_end(shape), shape);
@@ -413,9 +370,9 @@ namespace xt
      * iteration is broadcasted to the specified shape.
      * @param shape the shape used for braodcasting
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline auto xbroadcast<E, X, LV>::cxbegin(const S& shape) const noexcept -> xiterator<const_stepper, S>
+    inline auto xbroadcast<CT, X>::cxbegin(const S& shape) const noexcept -> xiterator<const_stepper, S>
     {
         // Could check if (broadcastable(shape, m_shape)
         return xiterator<const_stepper, S>(stepper_begin(shape), shape);
@@ -426,26 +383,26 @@ namespace xt
      * expression. The iteration is broadcasted to the specified shape.
      * @param shape the shape used for broadcasting
      */
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline auto xbroadcast<E, X, LV>::cxend(const S& shape) const noexcept -> xiterator<const_stepper, S>
+    inline auto xbroadcast<CT, X>::cxend(const S& shape) const noexcept -> xiterator<const_stepper, S>
     {
         // Could check if (broadcastable(shape, m_shape)
         return xiterator<const_stepper, S>(stepper_end(shape), shape);
     }
     //@}
 
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline auto xbroadcast<E, X, LV>::stepper_begin(const S& shape) const noexcept -> const_stepper
+    inline auto xbroadcast<CT, X>::stepper_begin(const S& shape) const noexcept -> const_stepper
     {
         // Could check if (broadcastable(shape, m_shape)
         return m_e.stepper_begin(shape);
     }
 
-    template <class E, class X, bool LV>
+    template <class CT, class X>
     template <class S>
-    inline auto xbroadcast<E, X, LV>::stepper_end(const S& shape) const noexcept -> const_stepper
+    inline auto xbroadcast<CT, X>::stepper_end(const S& shape) const noexcept -> const_stepper
     {
         // Could check if (broadcastable(shape, m_shape)
         return m_e.stepper_end(shape);
@@ -458,8 +415,8 @@ namespace xt
      * Returns an iterator to the first element of the buffer
      * containing the elements of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::storage_begin() const noexcept -> const_storage_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::storage_begin() const noexcept -> const_storage_iterator
     {
         return cbegin();
     }
@@ -468,8 +425,8 @@ namespace xt
      * Returns an iterator to the element following the last
      * element of the buffer containing the elements of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::storage_end() const noexcept -> const_storage_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::storage_end() const noexcept -> const_storage_iterator
     {
         return cend();
     }
@@ -478,8 +435,8 @@ namespace xt
      * Returns a constant iterator to the first element of the buffer
      * containing the elements of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::storage_cbegin() const noexcept -> const_storage_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::storage_cbegin() const noexcept -> const_storage_iterator
     {
         return cbegin();
     }
@@ -488,8 +445,8 @@ namespace xt
      * Returns a constant iterator to the element following the last
      * element of the buffer containing the elements of the expression.
      */
-    template <class E, class X, bool LV>
-    inline auto xbroadcast<E, X, LV>::storage_cend() const noexcept -> const_storage_iterator
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::storage_cend() const noexcept -> const_storage_iterator
     {
         return cend();
     }
