@@ -44,8 +44,9 @@ namespace xt
 
         using derived_type = D;
 
-        derived_type& derived_cast() noexcept;
-        const derived_type& derived_cast() const noexcept;
+        derived_type& derived_cast() & noexcept;
+        const derived_type& derived_cast() const & noexcept;
+        derived_type derived_cast() && noexcept;
 
     protected:
 
@@ -71,7 +72,7 @@ namespace xt
      * Returns a reference to the actual derived type of the xexpression.
      */
     template <class D>
-    inline auto xexpression<D>::derived_cast() noexcept -> derived_type&
+    inline auto xexpression<D>::derived_cast() & noexcept -> derived_type&
     {
         return *static_cast<derived_type*>(this);
     }
@@ -80,70 +81,133 @@ namespace xt
      * Returns a constant reference to the actual derived type of the xexpression.
      */
     template <class D>
-    inline auto xexpression<D>::derived_cast() const noexcept -> const derived_type&
+    inline auto xexpression<D>::derived_cast() const & noexcept -> const derived_type&
     {
         return *static_cast<const derived_type*>(this);
     }
+
+    /**
+     * Returns a constant reference to the actual derived type of the xexpression.
+     */
+    template <class D>
+    inline auto xexpression<D>::derived_cast() && noexcept->derived_type
+    {
+        return *static_cast<derived_type*>(this);
+    }
     //@}
 
-    template <class E>
-    using is_xexpression = std::is_base_of<xexpression<E>, E>;
+    namespace detail
+    {
+        template <class E>
+        struct is_xexpression_impl : std::is_base_of<xexpression<E>, E>
+        {
+        };
 
-    template <class E, class R>
+        template <class E>
+        struct is_xexpression_impl<xexpression<E>> : std::true_type
+        {
+        };
+    }
+
+    template <class E>
+    using is_xexpression = detail::is_xexpression_impl<E>;
+
+    template <class E, class R = void>
     using disable_xexpression = typename std::enable_if<!is_xexpression<E>::value, R>::type;
 
     template <class... E>
     using has_xexpression = or_<is_xexpression<E>...>;
 
-    /************************
-     * get_xexpression_type *
-     ************************/
+    /************
+     * xclosure *
+     ************/
 
     template <class T>
     class xscalar;
 
-    // Workaround for buggy error C2210 in VS2015 when using condtional_t
-    template <class E>
-    using get_xexpression_type = typename std::conditional<is_xexpression<typename std::remove_reference<E>::type>::value,
-                                                           typename std::remove_reference<E>::type, xscalar<E>>::type;
-
-    /*******************
-     * get_xexpression *
-     *******************/
-
-    template <class E>
-    inline const E& get_xexpression(const xexpression<E>& e) noexcept
+    namespace detail
     {
-        return e.derived_cast();
+        template <class S>
+        struct closure
+        {
+            using underlying_type = std::conditional_t<std::is_const<std::remove_reference_t<S>>::value,
+                                                       const std::decay_t<S>,
+                                                       std::decay_t<S>>;
+            using type = typename std::conditional<std::is_lvalue_reference<S>::value,
+                                                   underlying_type&,
+                                                   underlying_type>::type;
+        };
+
+        template <class S>
+        using closure_t = typename closure<S>::type;
+
+        template <class S>
+        struct const_closure
+        {
+            using underlying_type = const std::decay_t<S>;
+            using type = typename std::conditional<std::is_lvalue_reference<S>::value,
+                                                   underlying_type&,
+                                                   underlying_type>::type;
+        };
+        
+        template <class S>
+        using const_closure_t = typename const_closure<S>::type;
     }
 
-    template <class E>
-    inline disable_xexpression<E, xscalar<E>> get_xexpression(const E& e) noexcept
+    template <class E, class EN = void>
+    struct xclosure
     {
-        return xscalar<E>(e);
-    }
+        using type = detail::closure_t<E>;
+    };
 
-    /******************
-     * get_value_type *
-     ******************/
+    template <class E>
+    struct xclosure<E, disable_xexpression<std::decay_t<E>>>
+    {
+        using type = xscalar<detail::closure_t<E>>;
+    };
+
+    template <class E>
+    using xclosure_t = typename xclosure<E>::type;
+
+    template <class E, class EN = void>
+    struct const_xclosure
+    {
+        using type = detail::const_closure_t<E>;
+    };
+
+    template <class E>
+    struct const_xclosure<E, disable_xexpression<std::decay_t<E>>>
+    {
+        using type = xscalar<std::decay_t<E>>;
+    };
+ 
+    template <class E>
+    using const_xclosure_t = typename const_xclosure<E>::type;
+
+    /***************
+     * xvalue_type *
+     ***************/
 
     namespace detail
     {
         template <class E, class enable = void>
-        struct get_value_type_impl
+        struct xvalue_type_impl
         {
             using type = E;
         };
 
         template <class E>
-        struct get_value_type_impl<E, std::enable_if_t<is_xexpression<E>::value, void>>
+        struct xvalue_type_impl<E, std::enable_if_t<is_xexpression<E>::value>>
         {
             using type = typename E::value_type;
         };
     }
 
     template <class E>
-    using get_value_type = typename detail::get_value_type_impl<E>::type;
+    using xvalue_type = detail::xvalue_type_impl<E>;
+
+    template <class E>
+    using xvalue_type_t = typename xvalue_type<E>::type;
     
     /***************
      * get_element *
